{% extends "base.html" %}

{% block scripts %}
<!-- Google tag (gtag.js) -->
<script async src="https://www.googletagmanager.com/gtag/js?id=G-NJ1FSTXF78"></script>
<script>
    window.dataLayer = window.dataLayer || [];
    function gtag() { dataLayer.push(arguments); }
    gtag('js', new Date());

    gtag('config', 'G-NJ1FSTXF78');
</script>
<!-- Open API reference links in new tab -->
<script>
    document.addEventListener('DOMContentLoaded', function () {
        document.querySelectorAll('a[href*="/api/"]').forEach(function (link) {
            link.setAttribute('target', '_blank');
        });
    });
</script>
{{ super() }}
{% endblock %}
